<?php
namespace App\Utils;

use Symfony\Component\String\Slugger\AsciiSlugger;

class textComparatif {

    public static function jaccardSimilarity($str1, $str2) {
        // Utiliser l'AsciiSlugger pour convertir les chaînes de caractères en ensembles d'éléments distincts
        $slugger = new AsciiSlugger();
        $set1 = array_unique(str_split(strtolower($slugger->slug($str1))));
        $set2 = array_unique(str_split(strtolower($slugger->slug($str2))));
    
        // Calculer l'intersection des ensembles
        $intersection = count(array_intersect($set1, $set2));
    
        // Calculer l'union des ensembles
        $union = count($set1) + count($set2) - $intersection;
    
        // Calculer la similarité de Jaccard
        $similarity = 0;
        if($union){
            $similarity = $intersection / $union;
        }
    
        return $similarity;
    }

    public static function cosineDistance($text1, $text2) {
        // convertir les textes en vecteurs
        $vector1 = self::textToVector($text1);
        $vector2 = self::textToVector($text2);
        
        // calculer la distance cosinus entre les vecteurs
        $dotProduct = 0;
        foreach ($vector1 as $word => $count1) {
            if (isset($vector2[$word])) {
                $count2 = $vector2[$word];
                $dotProduct += $count1 * $count2;
            }
        }
        $magnitude1 = sqrt(array_sum(array_map(function($x) { return pow($x, 2); }, $vector1)));
        $magnitude2 = sqrt(array_sum(array_map(function($x) { return pow($x, 2); }, $vector2)));
        if ($magnitude1 == 0 || $magnitude2 == 0) {
            return 0;
        } else {
            return $dotProduct / ($magnitude1 * $magnitude2);
        }
    }
    
    public static function textToVector($text) {
        // prétraiter le texte pour enlever les caractères spéciaux et les espaces
        $text = strtolower(trim(preg_replace('/[^A-Za-z0-9\-]/', ' ', $text)));
        
        // convertir le texte en un tableau de mots
        $words = preg_split('/\s+/', $text);
        
        // compter le nombre d'occurrences de chaque mot
        $vector = array();
        foreach ($words as $word) {
            if (isset($vector[$word])) {
                $vector[$word]++;
            } else {
                $vector[$word] = 1;
            }
        }
        
        return $vector;
    }
}